package de.hft.stuttgart.citydoctor2.healing;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import de.hft.stuttgart.citydoctor2.check.HealingID;
import de.hft.stuttgart.citydoctor2.check.HealingMethod;
import de.hft.stuttgart.citydoctor2.check.ModificationListener;
import de.hft.stuttgart.citydoctor2.check.error.SolidNotClosedError;
import de.hft.stuttgart.citydoctor2.connect.edge.CppFeature;
import de.hft.stuttgart.citydoctor2.connect.edge.CppHealResult;
import de.hft.stuttgart.citydoctor2.connect.edge.CppPolygonHealing;
import de.hft.stuttgart.citydoctor2.datastructure.Geometry;


public class HealSolidNotClosedCpp implements HealingMethod {

	private static final Logger logger = LogManager.getLogger(HealSolidNotClosedCpp.class);
	
	@Override
	public HealingMethod createNew() {
		return new HealSolidNotClosedCpp();
	}
	
	@Override
	public boolean visit(SolidNotClosedError err, ModificationListener l) {
		logger.debug("Executing Repair for PolygonSameOrientationError");
		// Wenn auf CPP Seite implementiert dann das hier entfernen:
		if (HealSolidNotClosedError.fixVerticesCloseToEdges(err)) {
			// maybe errors are fixed don't try other fixes until next iteration
			return true;
		}
		// Bis hier
		
				
		// ----------------------------------------------
		// get Geometry and Geometry informations
		// ----------------------------------------------
		Geometry geometry = err.getGeometry();	
		String gmlId_string = geometry.getParent().getGmlId().toString();

		// ----------------------------------------------	
		// create feature and fill feature with geometry
		// ----------------------------------------------
		
		CppFeature feature = new CppFeature( gmlId_string);
		
//		MeshSurface surface = MeshSurface.of(geometry);
//		DebugUtils.printPolygon3d(surface.getPolygons().toArray(new CDPolygonNs[0]));
//		DebugUtils.printGeoknechtPolygon(surface.getPolygons().toArray(new CDPolygonNs[0]));
			
		feature.setGeometry( geometry );
		feature.setLoD(geometry.getLod());
		
		// ----------------------------------------------	
		// start healing method
		// ----------------------------------------------
		
		CppPolygonHealing cppPolygonHealing = new CppPolygonHealing(feature);
		cppPolygonHealing.createCppObject();
		
		CppHealResult healResult = cppPolygonHealing.healSolidNotClosed();
		
		if(healResult.isEmpty())
			return false;
		
		healResult.mergeIntoGeometry(geometry);
			
		// ----------------------------------------------	
		// dispose cpp Objects
		// ----------------------------------------------
		
		cppPolygonHealing.disposeCppObject();
		feature.disposeCppObject();

		return true;
	}

	@Override
	public HealingID getID() {
		return HealingID.S_NOT_CLOSED_CPP;
	}


}
